@@ -3,11 +3,13 @@ import os
 # Use 'final' as the 4th element to indicate
 # a full release
 
-VERSION = (0, 3, 6, 'final')
+VERSION = (0, 4, 1, 'final')
+
 
 def get_short_version():
     return '%s.%s' % (VERSION[0], VERSION[1])
 
+
 def get_version():
     version = '%s.%s' % (VERSION[0], VERSION[1])
     if VERSION[2]:
@@ -24,8 +26,8 @@ def get_version():
 # Eg: the template 'oscar/templates/oscar/base.html' can be accessed via both
 # 'base.html' and 'oscar/base.html'.  This allows Oscar's templates to be
 # extended by templates with the same filename
-OSCAR_MAIN_TEMPLATE_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)),
-                                       'templates/oscar')
+OSCAR_MAIN_TEMPLATE_DIR = os.path.join(
+    os.path.dirname(os.path.abspath(__file__)), 'templates/oscar')
 
 OSCAR_CORE_APPS = [
     'oscar',
@@ -53,7 +55,9 @@ OSCAR_CORE_APPS = [
     'oscar.apps.dashboard.offers',
     'oscar.apps.dashboard.ranges',
     'oscar.apps.dashboard.vouchers',
+    'oscar.apps.dashboard.communications',
     # 3rd-party apps that oscar depends on
+    'haystack',
     'treebeard',
     'sorl.thumbnail',
 ]
@@ -65,6 +69,7 @@ def get_core_apps(overrides=None):
     """
     if not overrides:
         return OSCAR_CORE_APPS
+
     def get_app_label(app_label, overrides):
         pattern = app_label.replace('oscar.apps.', '')
         for override in overrides:
