@@ -1,4 +1,4 @@
-from django.conf.urls.defaults import patterns, url
+from django.conf.urls import patterns, url
 from django.contrib.auth.decorators import login_required
 
 from oscar.apps.basket.views import (BasketView, SavedView,
@@ -18,11 +18,14 @@ class BasketApplication(Application):
         urlpatterns = patterns('',
             url(r'^$', self.summary_view.as_view(), name='summary'),
             url(r'^add/$', self.add_view.as_view(), name='add'),
-            url(r'^vouchers/add/$', self.add_voucher_view.as_view(), name='vouchers-add'),
-            url(r'^vouchers/(?P<pk>\d+)/remove/$', self.remove_voucher_view.as_view(), name='vouchers-remove'),
-            url(r'^saved/$', login_required(self.saved_view.as_view()), name='saved'),
+            url(r'^vouchers/add/$', self.add_voucher_view.as_view(),
+                name='vouchers-add'),
+            url(r'^vouchers/(?P<pk>\d+)/remove/$',
+                self.remove_voucher_view.as_view(), name='vouchers-remove'),
+            url(r'^saved/$', login_required(self.saved_view.as_view()),
+                name='saved'),
         )
         return self.post_process_urls(urlpatterns)
 
 
-application = BasketApplication()
\ No newline at end of file
+application = BasketApplication()
