@@ -3,11 +3,15 @@ from django.views.generic import ListView, DetailView
 from django.db.models import get_model
 from django.utils.translation import ugettext_lazy as _
 
+from oscar.core.loading import get_class
 from oscar.apps.catalogue.signals import product_viewed, product_search
 
 Product = get_model('catalogue', 'product')
 ProductReview = get_model('reviews', 'ProductReview')
 Category = get_model('catalogue', 'category')
+ProductAlert = get_model('customer', 'ProductAlert')
+ProductAlertForm = get_class('customer.forms',
+                             'ProductAlertForm')
 
 
 class ProductDetailView(DetailView):
@@ -15,33 +19,53 @@ class ProductDetailView(DetailView):
     model = Product
     view_signal = product_viewed
     template_folder = "catalogue"
-    _product = None
 
     def get_object(self):
-        if not self._product:
-            self._product = super(ProductDetailView, self).get_object()
+        # Use a cached version to prevent unnecessary DB calls
+        if not hasattr(self, '_product'):
+            setattr(self, '_product',
+                    super(ProductDetailView, self).get_object())
         return self._product
 
     def get_context_data(self, **kwargs):
         ctx = super(ProductDetailView, self).get_context_data(**kwargs)
-        ctx['reviews'] = ProductReview.objects.filter(status=ProductReview.APPROVED,
-                                                      product=self.object)
+        ctx['reviews'] = self.get_reviews()
+        ctx['alert_form'] = self.get_alert_form()
+        ctx['has_active_alert'] = self.get_alert_status()
         return ctx
 
+    def get_alert_status(self):
+        # Check if this user already have an alert for this product
+        has_alert = False
+        if self.request.user.is_authenticated():
+            alerts = ProductAlert.objects.filter(
+                product=self.object, user=self.request.user,
+                status=ProductAlert.ACTIVE)
+            has_alert = alerts.count() > 0
+        return has_alert
+
+    def get_alert_form(self):
+        return ProductAlertForm(user=self.request.user,
+                                product=self.object)
+
+    def get_reviews(self):
+        return self.object.reviews.filter(status=ProductReview.APPROVED)
+
     def get(self, request, **kwargs):
-        """
-        Ensure that the correct URL is used
-        """
+        # Ensure that the correct URL is used
         product = self.get_object()
         correct_path = product.get_absolute_url()
         if correct_path != request.path:
             return HttpResponsePermanentRedirect(correct_path)
-        response = super(ProductDetailView, self).get(request, **kwargs)
 
-        # Send signal to record the view of this product
-        self.view_signal.send(sender=self, product=product, user=request.user, request=request, response=response)
+        response = super(ProductDetailView, self).get(request, **kwargs)
+        self.send_signal(request, response, product)
         return response
 
+    def send_signal(self, request, response, product):
+        self.view_signal.send(sender=self, product=product, user=request.user,
+                              request=request, response=response)
+
     def get_template_names(self):
         """
         Return a list of possible templates.
@@ -74,6 +98,7 @@ def get_product_base_queryset():
         'product_options',
         'product_class__options',
         'stockrecord',
+        'stockrecord__partner',
         'images',
     ).all()
 
@@ -87,6 +112,9 @@ class ProductCategoryView(ListView):
     paginate_by = 20
 
     def get_categories(self):
+        """
+        Return a list of the current category and it's ancestors
+        """
         slug = self.kwargs['category_slug']
         try:
             category = Category.objects.get(slug=slug)
