@@ -1,4 +1,3 @@
-import datetime
 from decimal import Decimal as D
 
 from django.utils.translation import ugettext_lazy as _
@@ -8,7 +7,10 @@ class DefaultWrapper(object):
     """
     Default stockrecord wrapper
     """
-    
+    CODE_IN_STOCK = 'instock'
+    CODE_AVAILABLE = 'available'
+    CODE_UNAVAILABLE = 'outofstock'
+
     def is_available_to_buy(self, stockrecord):
         """
         Test whether a product is available to buy.
@@ -39,6 +41,8 @@ class DefaultWrapper(object):
         """
         Return the maximum available purchase quantity for a given user
         """
+        if not stockrecord.product.product_class.track_stock:
+            return None
         if stockrecord.num_in_stock is None:
             return None
         return stockrecord.net_stock_level
@@ -52,11 +56,11 @@ class DefaultWrapper(object):
         :param oscar.apps.partner.models.StockRecord stockrecord: stockrecord instance
         """
         if stockrecord.net_stock_level > 0:
-            return 'instock'
+            return self.CODE_IN_STOCK
         if self.is_available_to_buy(stockrecord):
-            return 'available'
-        return 'outofstock'
-    
+            return self.CODE_AVAILABLE
+        return self.CODE_UNAVAILABLE
+
     def availability(self, stockrecord):
         """
         Return an availability message for the passed stockrecord.
@@ -68,16 +72,20 @@ class DefaultWrapper(object):
         if self.is_available_to_buy(stockrecord):
             return _('Available')
         return _("Not available")
-    
+
     def dispatch_date(self, stockrecord):
-        if stockrecord.net_stock_level:
-            # Assume next day for in-stock items
-            return datetime.date.today() + datetime.timedelta(days=1)
-        # Assume one week for out-of-stock items
-        return datetime.date.today() + datetime.timedelta(days=7)
-    
+        """
+        We don't provide a default value as it could be confusing.  Subclass
+        and override this method to provide estimated dispatch dates
+        """
+        return None
+
     def lead_time(self, stockrecord):
-        return 1
-    
+        """
+        We don't provide a default value as it could be confusing.  Subclass
+        and override this method to provide estimated dispatch dates
+        """
+        return None
+
     def calculate_tax(self, stockrecord):
         return D('0.00')
