@@ -1,4 +1,5 @@
-from oscar.apps.voucher.abstract_models import AbstractVoucher, AbstractVoucherApplication
+from oscar.apps.voucher.abstract_models import (
+    AbstractVoucher, AbstractVoucherApplication)
 
 
 class Voucher(AbstractVoucher):
