@@ -6,8 +6,10 @@ from django.utils.translation import ugettext_lazy as _
 from oscar.core.loading import get_class
 
 ReportGenerator = get_class('dashboard.reports.reports', 'ReportGenerator')
-ReportCSVFormatter = get_class('dashboard.reports.reports', 'ReportCSVFormatter')
-ReportHTMLFormatter = get_class('dashboard.reports.reports', 'ReportHTMLFormatter')
+ReportCSVFormatter = get_class(
+    'dashboard.reports.reports', 'ReportCSVFormatter')
+ReportHTMLFormatter = get_class(
+    'dashboard.reports.reports', 'ReportHTMLFormatter')
 Voucher = get_model('voucher', 'Voucher')
 
 
@@ -41,8 +43,7 @@ class VoucherReportGenerator(ReportGenerator):
 
     formatters = {
         'CSV_formatter': VoucherReportCSVFormatter,
-        'HTML_formatter': VoucherReportHTMLFormatter
-    }
+        'HTML_formatter': VoucherReportHTMLFormatter}
 
     def generate(self):
         vouchers = Voucher._default_manager.all()
