@@ -1,4 +1,4 @@
-from django.conf.urls.defaults import patterns
+from django.conf.urls import patterns
 
 
 class Application(object):
@@ -12,7 +12,8 @@ class Application(object):
 
     def get_urls(self):
         """
-        Return the url patterns for this app, MUST be implemented in the subclass
+        Return the url patterns for this app, MUST be implemented in the
+        subclass
         """
         return patterns('')
 
@@ -24,8 +25,8 @@ class Application(object):
         could override this method to do anything you want.
         """
         for pattern in urlpatterns:
-            #if hasattr(pattern, 'url_patterns'):
-            #    self.post_process_urls(pattern.url_patterns)
+            if hasattr(pattern, 'url_patterns'):
+                self.post_process_urls(pattern.url_patterns)
             if not hasattr(pattern, '_callback'):
                 continue
             # Look for a custom decorator
