@@ -10,20 +10,24 @@ except ImportError:
     pass
 else:
     add_introspection_rules([], ["^oscar\.models\.fields\.ExtendedURLField$"])
-    add_introspection_rules([], ["^oscar\.models\.fields\.PositiveDecimalField$"])
+    add_introspection_rules([], [
+        "^oscar\.models\.fields\.PositiveDecimalField$"])
 
 
 class ExtendedURLField(CharField):
     description = _("URL")
 
-    def __init__(self, verbose_name=None, name=None, verify_exists=True, **kwargs):
+    def __init__(self, verbose_name=None, name=None,
+                 verify_exists=True, **kwargs):
         kwargs['max_length'] = kwargs.get('max_length', 200)
         CharField.__init__(self, verbose_name, name, **kwargs)
-        validator = validators.ExtendedURLValidator(verify_exists=verify_exists)
+        validator = validators.ExtendedURLValidator(
+            verify_exists=verify_exists)
         self.validators.append(validator)
 
     def formfield(self, **kwargs):
-        # As with CharField, this will cause URL validation to be performed twice
+        # As with CharField, this will cause URL validation to be performed
+        # twice.
         defaults = {
             'form_class': fields.ExtendedURLField,
         }
